private import responser;
private import response;
private import request;
private import bogoyaml;
private import typical_responses;
private import attohttpd;
private import log;
private import pipe;
private import fileutil;
private import myio;

private import std.string;
private import std.file;
private import std.c.stdio;
private import std.conv;

version (USE_CGI_RESPONSER) {

	extern (C) {
		FILE* popen(char* command, char* type);
		int pclose(FILE* fp);

	}

	version (linux) {
		extern (C) int chdir(char* path);
		extern (C) char* get_current_dir_name();
		extern (C) int setenv(char *name, char *value, int overwrite);
		extern (C) void unsetenv(char *name);
	}
	version (Win32) {
		private import std.c.windows.windows;
		extern (Windows) export BOOL SetCurrentDirectoryW(char* s);
		extern (Windows) export DWORD GetCurrentDirectoryW(int l, char* s);
		extern (Windows) export BOOL SetEnvironmentVariableW(char* k, char* v);
		int chdir(char* path) {
			SetCurrentDirectoryW(path);
			return 0;
		}
		char* get_current_dir_name() {
			static char dirbuf[1024];
			GetCurrentDirectoryW(1024, dirbuf);
			return dirbuf;
		}
		int setenv(char* k, char* v, int over) {
			SetEnvironmentVariableW(k, v);
			return 0;
		}
		void unsetenv(char *name) {
			SetEnvironmentVariableW(name, cast(char*)0);
		}
	}

	class CgiResponser : Responser {
		this() {
			YamlHolder conf = AttoHttpd.self.conf;
			suffixes_ = conf.get("CgiSuffix").array;

			curDir_ = get_current_dir_name();
		}

		~this() {
			std.c.stdlib.free(curDir_);
		}

		HttpResponse response(HttpRequest req) {
			char[] localFile = req.localFile;

            if (!req.isLocal) {
                return null;
            }

			bool isCgi = false;
			foreach (YamlHolder s; suffixes_) {
				if (s.str == req.suffix) {
					isCgi = true;
				}
			}

			if (!isCgi) return null;

			if (!file_canread(localFile)) {
				// ignoring 403...
				return TypicalResponses.resNotFound;
			}

			setenv("REQUEST_METHOD", std.string.toStringz(req.method), 1);

			setenv("QUERY_STRING", std.string.toStringz(req.queryString), 1);

//		if (req.queryString != "") {
//			setenv("QUERY_STRING", std.string.toStringz(req.queryString), 1);
//		}
//		else {
			// emulate GET method. but it doesn't work almost always...
/*
  setenv("REQUEST_METHOD", "GET", 1);
  setenv("QUERY_STRING", std.string.toStringz(req.messageBody), 1);
*/
//		}

			chdir(std.string.toStringz(req.dirName));

			try {

				auto Pipe p;
				if (req.messageBody == "") {
					unsetenv("CONTENT_LENGTH");
					p = new Pipe("./" ~ req.baseName, PipeMode.In);
				}
				else {
					setenv("CONTENT_LENGTH",
						   toStringz(tostr(req.messageBody.length)), 1);
					p = new Pipe("./" ~ req.baseName, PipeMode.InOut);
					p.writeBlock(req.messageBody, req.messageBody.length);
					//dout.writeLine(req.messageBody);
				}
				char[] all = "";
				char[1024] buf;
				int s;
				while ((s = p.readBlock(buf, buf.length)) == buf.length) {
					all ~= buf;
				}
				all ~= buf[0..s];
				p.close();

/*
  FILE* fp = popen(std.string.toStringz("./" ~ req.baseName), "r");
  char[] all = fp_read_all(fp);
  pclose(fp);
*/
				chdir(curDir_);

				int idx = std.string.find(all, "\r\n\r\n");
				int lbnum = 4;
				char[] delim = "\r\n";
				if (idx == -1) {
					// apache allows it.
					idx = std.string.find(all, "\n\n");
					lbnum = 2;
					delim = "\n";
				}

				if (idx == -1) {
					Log.warn("premature end of script headers: " ~ localFile);
					return TypicalResponses.resInternalServerError;
				}

				HttpResponse res = new HttpResponse("HTTP/1.0 200 OK");
				res.messageBody = all[idx+lbnum..all.length];

				foreach (char[] header; std.string.split(all[0..idx], delim)) {
					int idx = std.string.find(header, ":");
					if (idx == -1) {
						Log.warn("malformed header from script. Bad header=" ~
								 header ~ ": " ~ localFile);
						return TypicalResponses.resInternalServerError;
					}
					int wsidx = idx+1;
					while (header.length > wsidx && header[wsidx] == ' ') {
						wsidx++;
					}
					res.header[header[0..idx]] = header[wsidx..header.length];
				}

				res.addDefaultHeader();

				return res;
			}
			catch (Error e) {
				Log.warn(e.toString());
				return null;
			}
		}

	private:
		YamlHolder[] suffixes_;
		char* curDir_;
	}

}
