private import attohttpd;
private import socket;
private import myio;
private import request;
private import response;
private import log;
private import responser;
private import filter;
private import sockutil;

private import std.thread;

class Client : Thread {
public:
    this(Socket socket, Responser[] responsers, Filter[] filters) {
        socket_ = socket;
        responsers_ = responsers;
        filters_ = filters;
    }

    override int run() {
		static const int BUFSIZE = 1024 * 4;
		char buf[BUFSIZE];
		char[] data;

        if (socket_.receive(buf) <= 0) {
            close();
            return 0;
        }
        else {
            data ~= tostr(cast(char*)buf);
        }

		if (data.length == 0) {
            close();
            return 0;
        }

		HttpRequest req = new HttpRequest(data);

		Log.info("Request: " ~ req.statusLine);

		InternetAddress ip = cast(InternetAddress)socket_.remoteAddress();
        req.host = ip.addr;
        req.port = ip.port;

		foreach (Responser responser; responsers_) {
			HttpResponse res = responser.response(req);
			if (res !== null) {
				foreach (Filter f; filters_) {
					if (!f.filter(req, res)) break;
				}

				Log.info("Response(" ~ responser.classinfo.name ~
                         "): " ~ res.statusLine);

				char[] resStr = res.toString();
                safeSend(socket_, resStr);

                close();

				return 0;
			}
		}

		Log.warn("cannot response");

		close();

        return 0;
    }

    bit isTerminated() {
        return getState() == TS.TERMINATED;
    }

private:
	void close() {
		Log.info("connection closed");

        socket_.shutdown(SocketShutdown.BOTH);
        socket_.close();
	}

private:
    Socket socket_;
    Responser[] responsers_;
    Filter[] filters_;
}
