private import std.stream;
private import std.file;
private import std.string;
private import std.c.stdio;

private import log;

version (Win32) {
	private import std.c.windows.windows;
}
version (linux) {
	private import std.c.linux.linux;
}

bool file_canread(char[] name) {
	version (Win32) {
		uint ret = GetFileAttributesA(toStringz(name));
		if (ret == 0xFFFFFFFF) return false;
		return (ret & FILE_ATTRIBUTE_ARCHIVE) != 0;
	}
	version (linux) {
		int fd = open(toStringz(name), O_RDONLY);
		struct_stat st;
		fstat(fd, &st);
		close(fd);
		return (st.st_mode & S_IFREG) != 0;
	}
}

bool file_isdir(char[] name) {
	version (Win32) {
		uint ret = GetFileAttributesA(toStringz(name));
		if (ret == 0xFFFFFFFF) return false;
		return (ret & FILE_ATTRIBUTE_DIRECTORY) != 0;
	}
	version (linux) {
		static const int S_IFDIR = 0040000;
		int fd = open(toStringz(name), O_RDONLY);
		struct_stat st;
		fstat(fd, &st);
		close(fd);
		return (st.st_mode & S_IFDIR) != 0;
	}
}

version (USE_CGI_RESPONSER) {
/*
	extern (C) uint fread(void* ptr, uint size, uint nmemb, FILE* fp);

	char[] fp_read_all(FILE* fp) {
		char[256] buf;
		char[] ret;
		uint len;
		while ((len = fread(buf, 1, 256, fp)) > 0) {
			ret ~= buf[0..len];
		}
		return ret;
	}

	unittest {
		static const char[] TMP_FILE = "unittest.tmp";
		static const char[] LONG = "loooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooog";
		File o = new File(TMP_FILE, FileMode.Out);
		o.writeLine("hoge");
		o.writeLine(LONG);
		o.writeLine("");
		o.writeLine("hoge\r");
		o.close();

		FILE* fp = fopen(std.string.toStringz(TMP_FILE), "r");
		assert(fp_read_all(fp) == "hoge\n" ~ LONG ~ "\n" ~ "\n" ~ "hoge\r\n");
	}
*/
}

version (NULL) {
	extern(C) uint strlen(char* s);

	char[] fp_getline(FILE* fp) {
		char[256] buf;
		char* p = buf;
		char[] ret = "";
		int l;
		do {
			fgets(buf, 256, fp);
			l = strlen(p);
			ret ~= std.string.toString(p);
		} while (l == 255 && buf[254] != '\n');

		return ret;
	}

	unittest {
		static const char[] TMP_FILE = "unittest.tmp";
		static const char[] LONG = "loooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooog";
		File o = new File(TMP_FILE, FileMode.Out);
		o.writeLine("hoge");
		o.writeLine(LONG);
		o.writeLine("");
		o.writeLine("hoge\r");
		o.close();

		FILE* fp = fopen(std.string.toStringz(TMP_FILE), "r");
		assert(fp_getline(fp) == "hoge\n");
		assert(fp_getline(fp) == LONG ~ "\n");
		assert(fp_getline(fp) == "\n");
		assert(fp_getline(fp) == "hoge\r\n");
		assert(fp_getline(fp) == "");
	}
}
